﻿using System.Data;
using System.Data.SQLite;
using System.Reflection;

namespace TestProject;

/// <summary>
/// 프로그램
/// </summary>
class Program
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Private

    #region 리스트 구하기 - GetList<TData>(reader)

    /// <summary>
    /// 리스트 구하기
    /// </summary>
    /// <typeparam name="TData">데이터 타입</typeparam>
    /// <param name="reader">데이터 리더</param>
    /// <returns>리스트</returns>
    private static List<TData> GetList<TData>(IDataReader reader) where TData : new()
    {
        List<TData> list = new List<TData>();

        while(reader.Read())
        {
            TData data = new TData();

            for(int i = 0; i < reader.FieldCount; i++)
            {
                string fieldName = reader.GetName(i);

                object value = reader.GetValue(i);

                PropertyInfo propertyInfo = data.GetType().GetProperty(fieldName);

                if(propertyInfo != null && value != DBNull.Value)
                {
                    propertyInfo.SetValue(data, value);
                }
            }

            list.Add(data);
        }

        return list;
    }

    #endregion
    #region 프로그램 시작하기 - Main()

    /// <summary>
    /// 프로그램 시작하기
    /// </summary>
    private static void Main()
    {
        string connectionString = "URI=file:test.db";

        using(SQLiteConnection connection = new SQLiteConnection(connectionString))
        {
            connection.Open();

            SQLiteCommand command = new SQLiteCommand();

            command.Connection  = connection;
            command.CommandType = CommandType.Text;
            command.CommandText = "SELECT * FROM Sample";

            using(SQLiteDataReader reader = command.ExecuteReader())
            {
                List<Sample> sampleList = GetList<Sample>(reader);

                foreach(Sample sample in sampleList)
                {
                    Console.WriteLine(sample);
                }
            }
        }
    }

    #endregion
}